#include <stdio.h>
#include"Python.h"
#include"arrayobject.h"
#include"matfile.h"
 
/* only support 1d, 2d contiguous array */	    
extern PyArrayObject *transpose(PyArrayObject *ap)
{
    PyArrayObject *rt;
    int i, j, nd, dims[2];
    double *dp1, *dp2;
    Py_complex *cp1, *cp2;
    
    nd = ap->nd;
    if (nd > 2 || nd < 0) return 0;

    for (i=0; i<nd; i++)
	dims[i] = ap->dimensions[nd-i-1];
    rt = (PyArrayObject *)PyArray_FromDims(nd, dims,
					   ap->descr->type_num);
    if (ap->descr->type_num == PyArray_DOUBLE) {
	dp1 = (double *)rt->data;
	dp2 = (double *)ap->data;
	if (nd == 1)
	    for (i=0; i<dims[0]; i++)
		*(dp1+i) = *(dp2+i);
	else 
	    for (i=0; i<dims[0]; i++)
		for (j=0; j<dims[1]; j++)
		    *(dp1+i*dims[1]+j) = *(dp2+j*dims[0]+i);
    }
    else {
	cp1 = (Py_complex *)rt->data;
	cp2 = (Py_complex *)ap->data;
	if (nd == 1)
	    for (i=0; i<dims[0]; i++)
		*(cp1+i) = *(cp2+i);
	else
	    for (i=0; i<dims[0]; i++)
		for (j=0; j<dims[1]; j++)
		    *(cp1+i*dims[1]+j) = *(cp2+j*dims[0]+i);
    }
    return rt;
}

/* on failure, return 0 */
extern PyArrayObject *readMatFile(FILE *fp, const char* varname)
{
    Fmatrix hd;
    long LetterM, LetterO, LetterP, LetterT;
    int bpe, length, dims[2], typecode, i, j;
    PyArrayObject *ap, *ap1, *ap2;
    PyObject *op;
    char *name, *rbuff, *ibuff, *ribuff;

/* 
 * process header
 */  

    if (fread(&hd, sizeof(hd), 1, fp) != 1) {
	perror("Cannot read matfile header");
	return 0;
    }
    
    /* type */
    LetterM = hd.type / 1000;
    LetterO = (hd.type-1000*LetterM) / 100;
    LetterP = (hd.type-1000*LetterM-100*LetterO) / 10;
    LetterT = hd.type % 10;

    if (LetterM != 0 && LetterM != 1) {
	perror("Unsupported machine type");
	return 0;
    }

    if (LetterO != 0) {
	perror("Inconsistant matfile");
	return 0;
    }

    if (LetterP == 0) {
	bpe = sizeof(double);
	typecode = PyArray_DOUBLE;
    }
    else if (LetterP == 1) {
	bpe = 4*sizeof(char);
	typecode = PyArray_FLOAT;
    }
    else if (LetterP == 2) {
	bpe = 4*sizeof(char);
	typecode = PyArray_INT;
    }
    else if (LetterP == 3) {
	bpe = 2*sizeof(char);
	typecode = PyArray_SHORT; 
    }
    else if (LetterP == 4) {
	perror("No corresponding PyArray type for unsigned integer");
	return 0;
    }
    else if (LetterP == 5) {
	bpe = sizeof(char);
	typecode = PyArray_UBYTE;
    }
    else {
	perror("Bad storage code"); 
	return 0;
    }

    if (LetterT != 0) {
        perror("Unsupported matrix type");
	return 0;
    }

    /* nrows, ncols */
    if (hd.nrows <= 0 || hd.ncols <= 0) {
	perror("Negtive or zero dimension");
	return 0;
    }
    length = hd.nrows*hd.ncols;
    dims[0] = hd.nrows;
    dims[1] = hd.ncols;
    
    /* imagf */
    if (hd.imagf != 0 && hd.imagf != 1) {
	perror("Wrong imagf flag");
	return 0;
    }
    if (hd.imagf == 1 && LetterP != 0 && LetterP != 1) {
	perror("Inconsistant matfile");
	return 0;
    }

    /* namlen */
    if (hd.namlen <= 0) {
	perror("Negtive variable name length");
	return 0;
    }
    name = (char *)malloc(hd.namlen);
    if (fread(name, sizeof(char), hd.namlen, fp) != hd.namlen) {
	perror("Read name");
	return 0;
    }

    if (strcmp(name, varname) != 0) {
	perror("Variable name not right");
	return 0;
    }
    

/* 
 * Read matrix and assign to PyArray
 */

    /* read matrix */
    if ( !hd.imagf ) {
	rbuff = (char *)malloc(bpe*length);
	if (fread(rbuff, sizeof(char), bpe*length, fp) != bpe*length) {
	    perror("Read matrix");
	    free(rbuff);
	    return 0;
	}

	if (LetterM != MACHINE_ENDIAN) rbo(rbuff, bpe, length);

	ap = (PyArrayObject *)
	     PyArray_FromDimsAndData(2, dims, typecode, rbuff);  
    }
    else {
	rbuff = (char *)malloc(bpe*length);
	if (fread(rbuff, sizeof(char), bpe*length, fp) != bpe*length) {
	    perror("Read matrix");
	    free(rbuff);
	    return 0;
	}

	ibuff = (char *)malloc(bpe*length); 
	if (fread(ibuff, sizeof(char), bpe*length, fp) != bpe*length) {
	    perror("Read matrix");
	    free(ibuff);
	    return 0;
	}

	if (LetterM != MACHINE_ENDIAN) {
	    rbo(rbuff, bpe, length);
	    rbo(ibuff, bpe, length);
	}

	ribuff = (char *)malloc(bpe*length*2);
	typecode += 2;
	for (i=0; i<length; i++)
	    for (j=0; j<bpe; j++) {
		ribuff[2*i*bpe+j] = rbuff[i*bpe+j];
		ribuff[(2*i+1)*bpe+j] = ibuff[i*bpe+j];
	    }
	free(rbuff);
	free(ibuff);
	ap = (PyArrayObject *)
	     PyArray_FromDimsAndData(2, dims, typecode, ribuff);  
    }
    
/* 
 * COLUMN major -> ROW major
 */
    op = Py_BuildValue("[ii]", hd.ncols, hd.nrows);
    ap1 = (PyArrayObject *)PyArray_Reshape(ap, op);
    ap2 = transpose(ap1);
    Py_DECREF(ap);
    Py_DECREF(ap1);

    return ap2;
}


/* 
 * nd must be either 1 or 2.
 * type is casted into PyArray_DOUBLE or PyArray_CBOUBLE which are
 * the only supported formats by Octave
 * returns 0 on success, -1 on failure
 * It may overwrite or append depending on the open mode.
 */
extern int writeMatFile(FILE *fp, PyArrayObject *ap, const char *name)
{
    Fmatrix hd;
    double *rbuff, *ibuff, *dp;
    int length, bpe, i;
    PyArrayObject *ap1, *ap2, *ap3;
    
    if ( !PyArray_Check(ap) ) {
	perror("Not PyArrayObject");
	return -1;
    }
    if ( ap->nd > 2 ) {
	perror("Dimension bigger than 2");
	return -1;
    }

/* 
 * cast the data type and make it contiguous
 */
    if ( ap->descr->type_num <= PyArray_DOUBLE )
	ap1 = (PyArrayObject *)PyArray_Cast(ap, PyArray_DOUBLE);
    else if ( ap->descr->type_num <= PyArray_CDOUBLE )
	ap1 = (PyArrayObject *)PyArray_Cast(ap, PyArray_CDOUBLE);
    else {
	perror("Unsupported data type");
	return -1;
    }
    
    ap2 = (PyArrayObject *)
	PyArray_ContiguousFromObject(ap1, ap1->descr->type_num, 0, 0);
    Py_DECREF(ap1);
    

/*
 * Make up the header
 */
    
    hd.type = 1000 * MACHINE_ENDIAN;
    hd.nrows = ap2->dimensions[0];
    if ( ap2->nd == 2 ) hd.ncols = ap2->dimensions[1];
    else hd.ncols = 1;
    length = hd.nrows*hd.ncols;
    if ( ap2->descr->type_num == PyArray_DOUBLE ) hd.imagf = 0;
    else hd.imagf = 1;
    hd.namlen = strlen(name)+1;

/* 
 * write header, name
 */
   
    if (fwrite(&hd, sizeof(hd), 1, fp) != 1) {
 	perror("fwrite");
	return -1;
    }
    if (fwrite(name, sizeof(char), hd.namlen, fp) != hd.namlen ) {
 	perror("fwrite");
	return -1;
    }


/* 
 * ROW major -> COLUMN major
 */

    ap3 = transpose(ap2);
    Py_DECREF(ap2);
    
/* 
 * write matrix
 */

    bpe = sizeof(double);
    if ( !hd.imagf ) {
	if (fwrite((void *)ap3->data, (size_t)bpe, (size_t)length, fp) != length) {
 	    perror("fwrite");
	    return -1; 
	}
    }
    else { 
	rbuff = (double *)malloc(bpe*length);
	ibuff = (double *)malloc(bpe*length);
	dp = (double *)ap3->data;
	for (i=0; i<hd.nrows*hd.ncols; i++) {
	    rbuff[i] = *(dp+2*i);
            ibuff[i] = *(dp+2*i+1);
        }
	if (fwrite((void *)rbuff, bpe, length, fp) != length) {
 	    perror("fwrite");
	    free(rbuff);
	    free(ibuff);
	    return -1;
	}
	if (fwrite((void *)ibuff, bpe, length, fp) != length) {
 	    perror("fwrite");
	    free(rbuff);
	    free(ibuff);
	    return -1;
	}
	free(rbuff);
 	free(ibuff);
    }

    Py_DECREF(ap3);
    return 0;
}


/**********************************************************/
/*                                                        */
/*   SYNOPSIS: rbo(data, bpe, nel) ;                      */
/*             where:                                     */
/* 	    nel..... number of array elements             */
/* 	    data.... pointer to the first byte in the     */
/* 	             array                                */
/* 	    bpe..... bytes per array element              */
/*                                                        */
/*   PURPOSE: convert data from little to big endian (and */
/*            visa-versa)                                 */
/*                                                        */
/**********************************************************/
void rbo(char * data, int bpe, int nel) 
{
	int nswaps, i,j;		/* number of swaps to make per element */
	char tmp;			/* temporary storage for swapping      */
	long int p1, p2;		/* indexes for elements to be swapped  */
	
	nswaps = bpe / 2;  		/* divide element size by two          */
	if (nswaps == 0) return;	/* return if it is a byte array        */

	p1 = 0;
	for ( i=0; i<nel; i++) {
		p1 = i*bpe;
		p2 = p1 + bpe - 1;
		for (j=0; j<nswaps; j++) {
			tmp      = data[p1];
			data[p1] = data[p2];
			data[p2] = tmp;
			p1++;
			p2--;
		}
	}
	return;
}
