#include <octave/config.h>
#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>
#include <octave/help.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "oct-engine.h"

// DEFUN_DLD and the macros that it depends on are defined in the
// files defun-dld.h, defun.h, and defun-int.h.
DEFUN_DLD (mxsnd, args, nargout,
  "[errno] = mxsnd(msgid, val, [mtype])\n\
\n\
Use msgsnd to send a scalar, vector or matrix val with mtype mtype\n\
by message queue msgid.")
{    
    // The list of values to return.  See the declaration in oct-obj.h
    octave_value_list retval;

    // The arguments to this function are available in args.
    int nargin = args.length ();
    if (nargin < 2 || nargin > 3) {
	error ("need two or three arguments");
	return retval;
    }

    mxbuf buf;
    int msgid = (int)args(0).double_value();
    if (nargin == 3) buf.mtype =  (long)(args(2).double_value());
    else buf.mtype = 1;

    buf.major = COLUMN;
    buf.nd = 2;
    char *ptr;
    if (args(1).is_real_type ()) {
	Matrix m = args(1).matrix_value ();
	buf.type = PyArray_DOUBLE;
	buf.dims[0] = m.rows();
	buf.dims[1] = m.columns();
	MArray2<double> m1 = (MArray2<double>)(m);
	Array2<double> m2 = (Array2<double>)(m1);
	Array<double> m3 = (Array2<double>)(m2);
	ptr = (char *)m3.data();
	buf.size = sizeof(double)*buf.dims[0]*buf.dims[1];
    }
    else if (args(1).is_complex_type()) {
	ComplexMatrix m = args(1).complex_matrix_value ();
	buf.type = PyArray_CDOUBLE;
	buf.dims[0] = m.rows();
	buf.dims[1] = m.columns();
	MArray2<Complex> m1 = (MArray2<Complex>)(m);
	Array2<Complex> m2 = (Array2<Complex>)(m1);
	Array<Complex> m3 = (Array2<Complex>)(m2);
	ptr = (char *)m3.data();
	buf.size = sizeof(Complex)*buf.dims[0]*buf.dims[1];
    }
    
    int i = 0, offset = 0;
    while (1) {
	buf.data[i++] = *(ptr+(offset++));
	if (i == BUF_SIZE && offset != buf.size) {
	    printf("type=%d\n", buf.type);
	    if (msgsnd(msgid, (struct msgbuf *)&buf, sizeof(buf), 0)
		== -1) {
		error("mxsnd: can not msgsnd");
		return retval;
	    }
	    i = 0;
	}
	else if (offset == buf.size) {
	    printf("type=%d\n", buf.type);
	    for (int j=i; j<BUF_SIZE; j++)
		buf.data[j] = 0;
	    if (msgsnd(msgid, (struct msgbuf *)&buf, sizeof(buf), 0)
		== -1 ) { 
		error("mxsnd: can not msgsnd");
		return retval;
	    }
	    break;
	}
    }
    retval(0) = octave_value(0.);
    return retval;
}





