#ifndef OCT_ENGINE_H
#define OCT_ENGINE_H
#ifdef __cplusplus
    extern "C" {
#endif

#include"Python.h"
#include"arrayobject.h"

#define MIN(a, b) ((a) < (b) ? (a) : (b)) 
#define BUF_SIZE 1024
#define MAX_DIM 10
/* #define PROMPT "octave" */
#define PROMPT ">> "
#define FIFOBASE "/tmp/octave_fifo"

typedef struct 
{
    int cmd;			/* pipe to send command */
    int res;			/* pipe to retrieve response */
    char fifo[128];		/* the fifo filename*/
    char *out_buf;		/* output kept for eval_str */ 
    int out_buf_size;
} Engine;

#ifdef OCT_ENGINE
/* defined in multiarraymodule.c */
extern PyObject *PyArray_Transpose(PyArrayObject *ap, PyObject *op);

/*
 * Execute octave statement, return 0 on success, nonzero otherwise 
 */
extern int engEvalString(
	Engine	*ep,		/* engine pointer */
	const char *string	/* string for octave t execute */
	);


/*
 * Start remote octave process, return NULL on failure
 */
extern Engine *engOpen(
	const char *startcmd /* exec command string used to start octave */
	);


/*
 * Close down octave server, return 0 on success
 */
extern int engClose(
	Engine	*ep         /* engine pointer */
	);


/*
 * get a array from octaves workspace
 */
extern PyArrayObject *engGetArray(
        Engine	*ep,		/* engine pointer */
	const char *name	/* name of array to get */
	);


/*
 * put a array in octaves workspace, o success, -1 failure
 */
extern int engPutArray(
	Engine	*ep,		/* engine pointer */
	const char *name,	/* name of the variable put into octave */
	PyArrayObject *ap	/* array pointer */
	);


/*
 * register a buffer to hold octave text output
 */
extern int engOutputBuffer(
	Engine	*ep,		/* engine pointer */
	char	*buffer,	/* character array to hold output */
	int     buflen		/* buffer array length */
	);
#endif

#ifdef __cplusplus
    }	/* extern "C" */
#endif
#endif /* OCT_ENGINE_H */







