"""
Simple plotting interface using MATLAB. This module is really meant
to be nothing more than an example of how to use the PyMat module.
A true front-end for MATLAB plotting would have many more functions,
a class-based structure for figures with properties that could be
get/set, etc.
"""

import pyoct
import Numeric
import types
H = None

def openSession():
  global H
  
  if not H:
    H = pyoct.open()
    pyoct.echo(H, 128)
    #pyoct.eval(H, "use_plplot")

def closeSession():
  global H

  if H:
    pyoct.close(H)
    H=None

def eval(str):
  openSession()

  pyoct.eval(H, str)

def plot(x, y=None, *args):
  if not y:
    y=x
    x=Numeric.arrayrange(len(y))
  
  openSession()

  pyoct.put(H, 'x__', x)
  pyoct.put(H, 'y__', y)

  s = 'plot(x__,y__'
  for item in args:
    s = s + "," + repr(item)
  s = s + ')'

  pyoct.eval(H, s)

def hold(onoff):
  openSession()

  if onoff:
    s = 'hold on'
  else:
    s = 'hold off'

  pyoct.eval(H, s)

def subplot(rows,cols,index):
  openSession()

  s = 'subplot(%d,%d,%d)' % (rows,cols,index)
  pyoct.eval(H, s)

def close():
  openSession()

  pyoct.eval(H, 'close')

def newfigure():
  openSession()

  pyoct.eval(H, 'figure')

def figurenum():
  openSession()

  pyoct.eval(H, "h__=get(0,'CurrentFigure')")
  h = pyoct.get(H, 'h__')
  if len(h):
    return h[0]
  else:
    return None

def title(s, *args):
  openSession()

  s = s % args
  pyoct.eval(H, "title('%s')" % s)
  pyoct.eval(H, "replot")

def xlabel(s, *args):
  openSession()

  s = s % args
  pyoct.eval(H, "xlabel('%s')" % s)
  pyoct.eval(H, "replot")

def ylabel(s, *args):
  openSession()

  s = s % args
  pyoct.eval(H, "ylabel('%s')" % s)
  pyoct.eval(H, "replot")

def axis(val=None):
  openSession()

  if not val:
    pyoct.eval(H, 'h__ = axis')
    return pyoct.get(H, 'h__')

  if type(val) == types.StringType:
    pyoct.eval(H,"axis('%s')" % val)
    return

  pyoct.put(H, 'h__', val)
  pyoct.eval(H, 'axis(h__)')
  pyoct.eval(H, "replot")
  return

def text(x, y, z, s=None):
  openSession()

  if not s:
    s = z
    z = None

  if z:
    pyoct.eval(H, "text(%g,%g,%g,'%s')" % (x,y,z,s))
  else:
    pyoct.eval(H, "text(%g,%g,'%s')" % (x,y,s))
  pyoct.eval(H, "replot")
  return

